/*
 * ethernet_application.c
 *
 *  Created on: Jun 11, 2012
 *      Author: jan
 */

#include "stm32f2x7_eth.h"
#include "ethernet_application.h"

void Ethernet_GPIO_Config(void);
uint8_t Ethernet_MACDMA_Config(void);
uint32_t Ethernet_Link_PHYIT_Config(uint16_t PHYAddress);
void Ethernet_EXTI_Config(void);

__IO uint8_t EthLinkStatus = 0;

uint8_t Ethernet_Application_Config()
{
	/* Configure the GPIO ports for ethernet pins */
	Ethernet_GPIO_Config();

#ifdef MII_MODE
	SYSCFG_ETH_MediaInterfaceConfig(SYSCFG_ETH_MediaInterface_MII);
#elif defined RMII_MODE  // Mode RMII
	SYSCFG_ETH_MediaInterfaceConfig(SYSCFG_ETH_MediaInterface_RMII);
#endif

	/* Configure the Ethernet MAC/DMA */
	if(Ethernet_MACDMA_Config() != SUCCESS) return ERROR;

	/* Configure the PHY to generate an interrupt on change of link status */
	Ethernet_Link_PHYIT_Config(DP83848_PHY_ADDRESS);

	/* Configure the EXTI for Ethernet link status. */
	Ethernet_EXTI_Config();

	return SUCCESS;
}

void Ethernet_GPIO_Config(void)
{
	/* Ethernet pins configuration ************************************************/
	   /*									  Chinakit	Produkt
			MCO ------------------------------> PA8		N/A

			ETH_MII_TXD0/ETH_RMII_TXD0 -------> PB12	PB12
			ETH_MII_TXD1/ETH_RMII_TXD1 -------> PB13	PB13
			ETH_MII_RXD0/ETH_RMII_RXD0 -------> PC4		PC4
			ETH_MII_RXD1/ETH_RMII_RXD1 -------> PC5		PC5
			ETH_MII_TX_EN/ETH_RMII_TX_EN -----> PB11	PB11
			ETH_MII_RX_CLK/ETH_RMII_REF_CLK---> PA1/PA8	N/A
			ETH_MII_RX_DV/ETH_RMII_CRS_DV ----> PA7		PA7
	        ETH_MDIO -------------------------> PA2		PA2
	        ETH_MDC --------------------------> PC1		PC1
			INTERP ---------------------------> PB14	PE12   -> Eth_Link_EXTIConfig()
			ETH_MII_RX_ER --------------------> N/A		PE11*/

	GPIO_InitTypeDef GPIO_InitStructure;

	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB |
							RCC_AHB1Periph_GPIOC | RCC_AHB1Periph_GPIOI |
							RCC_AHB1Periph_GPIOG | RCC_AHB1Periph_GPIOH |
							RCC_AHB1Periph_GPIOF, ENABLE);

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;

#ifdef MII_MODE
	#ifdef PHY_CLOCK_MCO
		/* Configure MCO (PA8) */
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
		GPIO_Init(GPIOA, &GPIO_InitStructure);
		GPIO_PinAFConfig(GPIOA, GPIO_PinSource8, GPIO_AF_ETH);

		// Output HSE clock (25MHz) on MCO pin (PA8) to clock the PHY
		RCC_MCO1Config(RCC_MCO1Source_HSE, RCC_MCO1Div_1);
	#endif
#endif

	/* Configure PA1, PA2 and PA7 */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_7;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource1, GPIO_AF_ETH);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_ETH);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_ETH);

	/* Configure PB11, PB12 and PB13 */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource11, GPIO_AF_ETH);
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource12, GPIO_AF_ETH);
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource13, GPIO_AF_ETH);

	/* Configure PC1 (ETH_MDC) */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
	GPIO_Init(GPIOC, &GPIO_InitStructure);
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource1, GPIO_AF_ETH);

	/* Configure PC4 and PC5 */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_5;
	GPIO_Init(GPIOC, &GPIO_InitStructure);
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource4, GPIO_AF_ETH);
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource5, GPIO_AF_ETH);

	/* Configure PE11 (ETH_MII_RX_ER) */
	GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_11;
	GPIO_Init(GPIOE, &GPIO_InitStructure);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource11, GPIO_AF_ETH);

}

uint8_t Ethernet_MACDMA_Config(void)
{

	ETH_InitTypeDef ETH_InitStructure;

	// Reset ETHERNET on AHB Bus
	ETH_DeInit();

	// Software reset
	ETH_SoftwareReset();

	// Wait for software reset
	while (ETH_GetSoftwareResetStatus() == SET);

	// Enable ETHERNET clock
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_ETH_MAC | RCC_AHB1Periph_ETH_MAC_Tx |
							RCC_AHB1Periph_ETH_MAC_Rx, ENABLE);

	// ETHERNET Configuration --------------------------------------------------
	// Call ETH_StructInit if you don't like to configure all ETH_InitStructure parameter
	ETH_StructInit(&ETH_InitStructure);

	// Fill ETH_InitStructure parametrs
	//------------------------   MAC   -----------------------------------
	ETH_InitStructure.ETH_AutoNegotiation = ETH_AutoNegotiation_Enable;
	//ETH_InitStructure.ETH_AutoNegotiation = ETH_AutoNegotiation_Disable;
	//  ETH_InitStructure.ETH_Speed = ETH_Speed_10M;
	//  ETH_InitStructure.ETH_Mode = ETH_Mode_FullDuplex;

	ETH_InitStructure.ETH_LoopbackMode = ETH_LoopbackMode_Disable;
	ETH_InitStructure.ETH_RetryTransmission = ETH_RetryTransmission_Disable;
	ETH_InitStructure.ETH_AutomaticPadCRCStrip = ETH_AutomaticPadCRCStrip_Disable;
	ETH_InitStructure.ETH_ReceiveAll = ETH_ReceiveAll_Disable;
	ETH_InitStructure.ETH_BroadcastFramesReception = ETH_BroadcastFramesReception_Enable;
	ETH_InitStructure.ETH_PromiscuousMode = ETH_PromiscuousMode_Disable;
	ETH_InitStructure.ETH_MulticastFramesFilter = ETH_MulticastFramesFilter_Perfect;
	ETH_InitStructure.ETH_UnicastFramesFilter = ETH_UnicastFramesFilter_Perfect;
#ifdef CHECKSUM_BY_HARDWARE
	ETH_InitStructure.ETH_ChecksumOffload = ETH_ChecksumOffload_Enable;
#endif

	//------------------------   DMA   -----------------------------------

	// When we use the Checksum offload feature, we need to enable the Store and Forward mode:
	//the store and forward guarantee that a whole frame is stored in the FIFO, so the MAC can insert/verify the checksum,
	//if the checksum is OK the DMA can handle the frame otherwise the frame is dropped
	ETH_InitStructure.ETH_DropTCPIPChecksumErrorFrame = ETH_DropTCPIPChecksumErrorFrame_Enable;
	ETH_InitStructure.ETH_ReceiveStoreForward = ETH_ReceiveStoreForward_Enable;
	ETH_InitStructure.ETH_TransmitStoreForward = ETH_TransmitStoreForward_Enable;

	ETH_InitStructure.ETH_ForwardErrorFrames = ETH_ForwardErrorFrames_Disable;
	ETH_InitStructure.ETH_ForwardUndersizedGoodFrames = ETH_ForwardUndersizedGoodFrames_Disable;
	ETH_InitStructure.ETH_SecondFrameOperate = ETH_SecondFrameOperate_Enable;
	ETH_InitStructure.ETH_AddressAlignedBeats = ETH_AddressAlignedBeats_Enable;
	ETH_InitStructure.ETH_FixedBurst = ETH_FixedBurst_Enable;
	ETH_InitStructure.ETH_RxDMABurstLength = ETH_RxDMABurstLength_32Beat;
	ETH_InitStructure.ETH_TxDMABurstLength = ETH_TxDMABurstLength_32Beat;
	ETH_InitStructure.ETH_DMAArbitration = ETH_DMAArbitration_RoundRobin_RxTx_2_1;

	// Configure Ethernet
	return (ETH_Init(&ETH_InitStructure, DP83848_PHY_ADDRESS) == 0 ? ERROR : SUCCESS);
}



uint32_t Ethernet_Link_PHYIT_Config(uint16_t PHYAddress)
{
/*
	uint32_t tmpreg = 0;

	// Read MICR register
	tmpreg = ETH_ReadPHYRegister(PHYAddress, PHY_MICR);

	// Enable output interrupt events to signal via the INT pin
	tmpreg |= (uint32_t)PHY_MICR_INT_EN | PHY_MICR_INT_OE;
	if(!(ETH_WritePHYRegister(PHYAddress, PHY_MICR, tmpreg)))
	{
		// Return ERROR in case of write timeout
		return ETH_ERROR;
	}

	// Read MISR register
	tmpreg = ETH_ReadPHYRegister(PHYAddress, PHY_MISR);

	// Enable Interrupt on change of link status
	tmpreg |= (uint32_t)PHY_MISR_LINK_INT_EN;
	if(!(ETH_WritePHYRegister(PHYAddress, PHY_MISR, tmpreg)))
	{
		// Return ERROR in case of write timeout
		return ETH_ERROR;
	}
	// Return SUCCESS
*/
	return ETH_SUCCESS;
}


void Ethernet_EXTI_Config(void)
{
/*

	GPIO_InitTypeDef GPIO_InitStructure;
	EXTI_InitTypeDef EXTI_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	// Enable the INT (PB14) Clock
	RCC_AHB1PeriphClockCmd(ETH_LINK_GPIO_CLK, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

	// Configure INT pin as input
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Pin = ETH_LINK_PIN;
	GPIO_Init(ETH_LINK_GPIO_PORT, &GPIO_InitStructure);

	// Connect EXTI Line to INT Pin
	SYSCFG_EXTILineConfig(ETH_LINK_EXTI_PORT_SOURCE, ETH_LINK_EXTI_PIN_SOURCE);

	// Configure EXTI line
	EXTI_InitStructure.EXTI_Line = ETH_LINK_EXTI_LINE;
	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
	EXTI_Init(&EXTI_InitStructure);

	// Enable and set the EXTI interrupt to the highest priority
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
	NVIC_InitStructure.NVIC_IRQChannel = EXTI15_10_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
*/
}

void EXTI15_10_IRQHandler(void)
{
	if(EXTI_GetITStatus(ETH_LINK_EXTI_LINE) != RESET)
	{
		if(((ETH_ReadPHYRegister(DP83848_PHY_ADDRESS, PHY_MISR)) & PHY_LINK_STATUS) != 0)
		{
			EthLinkStatus = ~EthLinkStatus;
		}
		/* Clear interrupt pending bit */
		EXTI_ClearITPendingBit(ETH_LINK_EXTI_LINE);
	}
}
